using Test
using SafeTestsets

using HydrophoneCalibrations
using HydrophoneCalibrations.Unitful

tolerance_v2p = 1e4u"Pa/V";


@testset "HydrophoneCalibrations.jl" begin
    @testset "precision acoustics hydrophone" begin
        v2p = volt_to_pressure(1e6, :PA_3197)
        sensitivity_at_1mhz = 34u"mV/MPa"
        conversion_factor = uconvert(u"Pa/V", 1/sensitivity_at_1mhz)
        @test isapprox(v2p, conversion_factor, atol=tolerance_v2p)
    end

    @testset "onda hydrophone" begin
        v2p = volt_to_pressure(1e6, :Onda_HGL0200_2322)
        sensitivity_at_1mhz = 4.11e-8u"V/Pa"
        conversion_factor = uconvert(u"Pa/V", 1/sensitivity_at_1mhz)
        @test isapprox(v2p, conversion_factor, atol=tolerance_v2p)
    end

    @testset "onda hydrophone with preamplifier (linear calculations)" begin
        v2p = volt_to_pressure(1e6, :Onda_HGL0200_2322, :Onda_AH2020_1238_20dB)
        sensitivity_at_1mhz = 4.11e-8u"V/Pa"
        hydrophone_capacitance_at_1mhz = 9.963u"pF"
        preamp_capacitance_at_1mhz = 7.08u"pF"
        preamp_gain_at_1mhz = 19.92u"dB"
        capacitance_factor = hydrophone_capacitance_at_1mhz/(hydrophone_capacitance_at_1mhz+preamp_capacitance_at_1mhz)
        system_sensitivity = uconvertrp(NoUnits, preamp_gain_at_1mhz) * sensitivity_at_1mhz * capacitance_factor
        conversion_factor = uconvert(u"Pa/V", 1/system_sensitivity)
        @test isapprox(v2p, conversion_factor, atol=tolerance_v2p)
    end

    @testset "onda hydrophone with preamplifier (logarithmic calculations)" begin
        v2p = volt_to_pressure(1e6, :Onda_HGL0200_2322, :Onda_AH2020_1238_20dB)
        sensitivity_at_1mhz = 4.11e-8u"V/Pa"
        hydrophone_capacitance_at_1mhz = 9.963u"pF"
        preamp_capacitance_at_1mhz = 7.08u"pF"
        preamp_gain_at_1mhz = 19.92u"dB"
        capacitance_factor = hydrophone_capacitance_at_1mhz/(hydrophone_capacitance_at_1mhz+preamp_capacitance_at_1mhz)
        system_sensitivity = preamp_gain_at_1mhz + ustrip(uconvertrp(u"dBV/Pa", sensitivity_at_1mhz))*u"dBV" + uconvertrp(u"dB", capacitance_factor)
        system_sensitivity = uconvertrp(u"V", system_sensitivity) / u"Pa"
        conversion_factor = uconvert(u"Pa/V", 1/system_sensitivity)
        @test isapprox(v2p, conversion_factor, atol=tolerance_v2p)
    end
end
